@@ -1,10 +1,9 @@
 /*
  * Linux USB support
  *
- * Written by:
- * Copyright 2000, Johannes Erdfelt <jerdfelt@valinux.com>
+ * Copyright (c) 2000-2001 Johannes Erdfelt <johannes@erdfelt.com>
  *
- * This library is covered by the LGPL.
+ * This library is covered by the LGPL, read LICENSE for details.
  */
 
 #include <stdlib.h>	/* getenv, etc */
@@ -212,7 +211,7 @@ int usb_find_devices_on_bus(struct usb_bus *bus)
   while ((entry = readdir(dir)) != NULL) {
     char filename[PATH_MAX + 1];
     struct usb_device *dev;
-    int fd, ret;
+    int fd, ret, skip = 0;
 
     /* Skip anything starting with a . */
     if (entry->d_name[0] == '.')
@@ -220,10 +219,15 @@ int usb_find_devices_on_bus(struct usb_bus *bus)
 
     /* Don't readd it if we were called multiple times */
     for (dev = bus->devices; dev; dev = dev->next) {
-      if (!strcmp(dev->filename, entry->d_name))
-        continue;
+      if (!strcmp(dev->filename, entry->d_name)) {
+        skip = 1;
+        break;
+      }
     }
 
+    if (skip)
+      continue;
+
     dev = malloc(sizeof(*dev));
     if (!dev)
       USB_ERROR(-ENOMEM);
@@ -286,6 +290,7 @@ int usb_find_busses(void)
 
   while ((entry = readdir(dir)) != NULL) {
     struct usb_bus *bus;
+    int skip = 0;
 
     /* Skip anything starting with a . */
     if (entry->d_name[0] == '.')
@@ -300,10 +305,15 @@ int usb_find_busses(void)
 
     /* Don't readd it if we were called multiple times */
     for (bus = usb_busses; bus; bus = bus->next) {
-      if (!strcmp(bus->dirname, entry->d_name))
-        continue;
+      if (!strcmp(bus->dirname, entry->d_name)) {
+        skip = 1;
+        break;
+      }
     }
 
+    if (skip)
+      continue;
+
     bus = malloc(sizeof(*bus));
     if (!bus)
       USB_ERROR(-ENOMEM);
