@@ -1,10 +1,9 @@
 /*
  * Main API entry point
  *
- * Written by:
- * Copyright 2000, Johannes Erdfelt <jerdfelt@valinux.com>
+ * Copyright (c) 2000-2001 Johannes Erdfelt <johannes@erdfelt.com>
  *
- * This library is covered by the LGPL.
+ * This library is covered by the LGPL, read LICENSE for details.
  */
 
 #include <stdlib.h>	/* getenv */
@@ -21,9 +20,24 @@ int usb_find_devices(void)
   int ret;
 
   for (bus = usb_busses; bus; bus = bus->next) {
+    struct usb_device *dev;
+
     ret = usb_find_devices_on_bus(bus);
     if (ret < 0)
       return ret;
+
+    /* Fixup the descriptor endianess */
+    for (dev = bus->devices; dev; dev = dev->next) {
+      usb_dev_handle *udev;
+
+      udev = usb_open(dev);
+      if (!udev)
+        continue;
+
+      usb_fetch_descriptors(udev);
+
+      usb_close(udev);
+    }
   }
 
   return 0;
